/*
 * IO2.h
 * 
 * Copyright 2013 OLIMEX LTD/Stefan Mavrodiev <support@olimex.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 *
 * Updated for the new firmware of MOD-IO2 (4.3)
 * 2022/05/13 - Stanimir Petev
 */

#ifndef IO2_H
#define IO2_H

#include <inttypes.h>

#define DEFAULT_ADDRESS   0x21

#define IN    1
#define OUT   0
#define LO    0
#define HI    1
#define ON    1
#define OFF   0

#define RELAY1   0x01
#define RELAY2   0x02

#define GPIO0    0x01
#define GPIO1    0x02
#define GPIO2    0x04
#define GPIO3    0x08
#define GPIO4    0x10
#define GPIO5    0x20
#define GPIO6    0x40

#define AN0   0x10
#define AN1   0x11
#define AN2   0x12
#define AN3   0x13
#define AN5   0x15

#define SET_RELAY     0x40
#define SET_GPIO2_DAC 0x60
#define SET_ADDRESS   0xF0

#define SET_TRIS   0x01
#define SET_LAT    0x02
#define GET_PORT   0x03
#define SET_PU     0x04
#define GET_TRIS   0x05
#define GET_LAT    0x06

#define GET_ID     0x20
#define GET_REV    0x21


class IO2
{
	private:
		uint8_t address;
		uint8_t relay_status;
		uint8_t tris_status;
		uint8_t lat_status;
		uint8_t pp_status;
	public:
		IO2();
		IO2(uint8_t);
		void setRelay(uint8_t relay, uint8_t state);
		void setAddress(uint8_t newAddress);
		void pinMode(uint8_t pin, uint8_t mode);
		void digitalWrite(uint8_t pin, uint8_t level);
		uint8_t digitalRead(uint8_t pin);
		uint16_t analogRead(uint8_t pin);
		void setPullpin(uint8_t pin, uint8_t state);
		uint8_t readID();
		uint8_t readRevision();
		void setGPIO2_DAC (int level);
};
#endif
